#!/usr/bin/perl -w
# ------------------------------------------------------------------
# RATO (version 1.3)
# ------------------------------------------------------------------
# --  A utility to extract different ratograms from multidivtime
#     output.
#
#  -- v1.1 calculates the average rate of all branches and writes
#     it together with the ratogram to a NEXUS treefile
#     (14.07.2004, Frank Rutschmann, frank@plant.ch).
#
#  -- v1.2 works now with multiple genes: calculates ratograms
#     for every single gene plus a ratogram averaged over all genes.
#     (16.12.2005, Frank Rutschmann, frank@plant.ch).
#
#  -- v1.3 now reads rates output from different multidivtime versions
#     and is able to handle data from more than 10 genes
#     (17.12.2005, Frank Rutschmann, frank@plant.ch).
#
#
# USAGE:
# [perl] rato.pl name_of_multidivtime_output_file
#
# Copyright (C) 2004 Torsten Eriksson
# Torsten.Eriksson@Bergianska.se
# This program is free software; 
# you can redistribute it and/or modify it under the terms of the 
# GNU General Public License as published by the Free Software Foundation; 
# either version 2 of the License, or (at your option) any later version. 
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# http://www.gnu.org/copyleft/gpl.html
#
# --------------------------------------------
use strict;
#
# ----------------
# Define variables
# ----------------
my $usage = "[perl] rato.pl name_of_the_multidivtime_output_file";
my $version = "\nWelcome! This is RATO v1.3 (December 17th, 2005)\n------------------------------------------------\n";

my $infile;             # Handle for input file
my $outfile;		        # Handle for output file
my $master_tree;        # Input chronogram
my @thedata;            # Array where input data are entered
my $ratogram;           # Output ratogram

my $tree_now = 0;       # Set tree indicator to FALSE
my $rates_now = 0;      # Set rates indicator to FALSE

my @rate;               # Rate at a specific node of a certain gene
my @allrate;            # Rate at a specific node of all genes
my $nodename;           # Current node/branch
my $gene = 0;           # Current gene
my @genename;           # Current gene name
my @noderate;           # Temporal rate variable at a certain node of a certain gene

my @totalrate;          # Total rate of all nodes of a certain gene
my $alltotalrate = 0;   # Total rate of all nodes of all genes
my @avrate = 0;         # Average rate of all nodes of a certain gene
my $allavrate = 0;      # Average rate of all nodes of all genes

my $numgenes = 0;       # Number of genes to be analyzed: recognized by the script
my @numnodes;           # Number of nodes to be analyzed per gene: recognized by the script
my $cnt;                # Counter


# -------------------
# Open the input file
# -------------------
if (not defined($ARGV[0])) {die "Error: Missing file name\nUsage: $usage\n";}
print $version . "\n";
$infile = $ARGV[0]; # name of input file
chomp ($infile);
open (TF,$infile) || die "Error: Unable to open the file: $!";
$_ = $infile;
/\./;
$outfile = "ratogram." . $' . ".tre"; # fix output file name
print "Analyzing input file " . $infile . ":\n";

# --------------------
# Fetch data from file
# --------------------
@thedata = <TF>;
close (TF) || warn "Error when closing data file: $!";

foreach(@thedata)
{
  chomp;
  if (/Name of gene/)
  {
  s/.*is //;
  $genename[$numgenes] = $_;
  print "Detected gene " . $numgenes . ": " . $genename[$numgenes] . "\n";
  $numgenes++;
  }
  if ($tree_now)
  {
    # This line contains master tree
    #(((((3Dact:0,((19Cpan:1,(2Cborn:2,(1Cgri:3,16Cglab:4):134):135):136,(17Axz:5,32Axc:6):133):137):138,(12Alz:7,((4Rhyn:8,((50Olrad:9,(14Olinv:10,49Olcap:11):127):128,(13Oline:12,51Olvang:13):126):129):130,(11Endo:14,(54Endlat:15,(55Glisch:16,(53Brup:17,(((65Stylfr:18,(9Style:19,18Bfuc:20):118):119,((56Penacut:21,(58Pengig:22,(60Penovat:23,(59Penlanc:24,(10Pen:25,(57Pencn:26,61Penrusc:27):111):112):113):114):115):116,(20Bmic:28,(62Pendahl:29,64Styleri:30):109):110):117):120,((21Stylm:31,5Bac:32):107,(52Bmund:33,(22Salt:34,(23Sond:35,63Sondsp:36):104):105):106):108):121):122):123):124):125):131):132):139,((6Memd:37,45Meme:38):102,((44Ptere:39,24Pterc:40):100,((46Rhexvi:41,(7Mela:42,G5Tibou:43):97):98,(G11Bertol:44,((25Cli:45,(33Tococa:46,34Miconia:47):93):94,(36Meri:48,(35Graff:49,43Macro:50):91):92):95):96):99):101):103):140,(((37Lopho:51,((8Trist:52,(G12Eugen:53,39Uro:54):85):86,(G1Metro:55,((G3Callist:56,G7Melaleu:57):82,((G4Eucal:58,38Ango:59):80,(G13Kunzea:60,G6Lepto:61):79):81):83):84):87):88,(29Psilox:62,30Hetero:63):78):89,(27Vochysi:64,28Ruiz:65):77):90):141,(G15Ludwig:66,((G10Gaura:67,G9Oeno:68):74,(48Circaea:69,(47Fuchspr:70,G8Fuchsia:71):72):73):75):76);

    $master_tree = $_;
    $tree_now=0; # set indicator to false
  }
  elsif (/^Here are node numbers of other internal nodes on master tree/)
  {
    # We found the line before master tree
    # Here are node numbers of other internal nodes on master tree:
    $tree_now = 1; # set indicator to true
  }
  elsif ($rates_now && /Rate Change/)
  {
    #Rate Change[Gene  0] =    0.04634 (S.D. =   0.02719) (  0.01218,  0.11841)
    last;
  }
}
print "Data for $numgenes genes has been found in the input file $infile.\n";
print "Writing output to the NEXUS tree file $outfile.\n";

# ---------------------------------
# Write initial part of output file
# ---------------------------------
open(OF,">$outfile") || die "Error: Could not open output file\n";
print OF "#NEXUS\n";
print OF "begin trees;\n";

# -------------------------
# Read rates for every gene
# -------------------------
foreach $gene (0..$numgenes-1)
{
  print "\nGene " . $gene . " (" . $genename[$gene] . ")\n------------------------\n";
  $totalrate[$gene] = 0;
  $numnodes[$gene] = 0;

  # --------------------------------------------
  # Read rates for every node of a specific gene
  # --------------------------------------------
  foreach(@thedata)
  {
    chomp;

    if (/Rate\[Gene  $gene,Node/ || /Rt\[Gene  $gene,Node/ || /Rt\[Gene $gene,Node/)
    {
      # Pick up rates
      #Rate[Gene  0,Node  0]=    0.00145 (S.D. =   0.00068) (  0.00044,  0.00306)
      @_ = split/[ ]+/;
      $_ = $_[2];
      /\d+/;
      $nodename = $&;
      $_ = $_[3];
      /[\d\.]+/;
      $noderate[$gene] = $&;
      #print "Gene " . $gene . ", node " . $nodename . ", rate " . $noderate[$gene] . "\n";
      $rate[$gene][$nodename]=$noderate[$gene];
      $totalrate[$gene] = $totalrate[$gene] + $noderate[$gene];
      $numnodes[$gene] = $numnodes[$gene] + 1;
    }
  }

  if ($numnodes[$gene] == 0)
  {
    die "Error: No rates found for gene $gene in $infile" . "!\n";
  }


  # ----------------------------------------------------
  # Scan master tree and exchange node numbers for rates
  # ----------------------------------------------------
  # This line contains master tree
  #(((((3Dact:0,((19Cpan:1,(2Cborn:2,(1Cgri:3,16Cglab:4):134):135):136,(17Axz:5,32Axc:6):133):137):138,(12Alz:7,((4Rhyn:8,((50Olrad:9,(14Olinv:10,49Olcap:11):127):128,(13Oline:12,51Olvang:13):126):129):130,(11Endo:14,(54Endlat:15,(55Glisch:16,(53Brup:17,(((65Stylfr:18,(9Style:19,18Bfuc:20):118):119,((56Penacut:21,(58Pengig:22,(60Penovat:23,(59Penlanc:24,(10Pen:25,(57Pencn:26,61Penrusc:27):111):112):113):114):115):116,(20Bmic:28,(62Pendahl:29,64Styleri:30):109):110):117):120,((21Stylm:31,5Bac:32):107,(52Bmund:33,(22Salt:34,(23Sond:35,63Sondsp:36):104):105):106):108):121):122):123):124):125):131):132):139,((6Memd:37,45Meme:38):102,((44Ptere:39,24Pterc:40):100,((46Rhexvi:41,(7Mela:42,G5Tibou:43):97):98,(G11Bertol:44,((25Cli:45,(33Tococa:46,34Miconia:47):93):94,(36Meri:48,(35Graff:49,43Macro:50):91):92):95):96):99):101):103):140,(((37Lopho:51,((8Trist:52,(G12Eugen:53,39Uro:54):85):86,(G1Metro:55,((G3Callist:56,G7Melaleu:57):82,((G4Eucal:58,38Ango:59):80,(G13Kunzea:60,G6Lepto:61):79):81):83):84):87):88,(29Psilox:62,30Hetero:63):78):89,(27Vochysi:64,28Ruiz:65):77):90):141,(G15Ludwig:66,((G10Gaura:67,G9Oeno:68):74,(48Circaea:69,(47Fuchspr:70,G8Fuchsia:71):72):73):75):76);

  $_ = $master_tree;
  my $key;
  foreach $key (0..$numnodes[$gene]-1)
  {
    #print $rate[$gene][$key] . "\n";
    s/(\:)($key)([,\)])/\:$rate[$gene][$key]$3/
  }

  $ratogram = $_;
  $avrate[$gene] = $totalrate[$gene] / $numnodes[$gene];

  #print $master_tree . "\n\n";
  print "Ratogram for gene $gene" . " (" . $genename[$gene] . "):\n" . $ratogram . "\n\n";
  print "The number of branches is $numnodes[$gene], the average rate across the tree is $avrate[$gene].\n";
  print OF "tree gene_$gene" . "_" . $genename[$gene] . "_$avrate[$gene] = [&R] $ratogram\n";
  print "The ratogram for gene $gene" . " (" . $genename[$gene] . ") has been written to the NEXUS file $outfile.\n\n";
}

# ------------------------------------
# Calculate average rates across genes
# ------------------------------------
print "\nAll genes" . "\n---------\n";
foreach $nodename (0..$numnodes[0]-1)
{
  $allrate[$nodename] = 0;
  foreach $gene (0..$numgenes-1)
  {
    $allrate[$nodename] = $allrate[$nodename] + $rate[$gene][$nodename];
  }
  $allrate[$nodename] = $allrate[$nodename] / $numgenes;
  #print "Average rate over all $numgenes genes for node " . $nodename . ": " . $allrate[$nodename] . "\n";
  $alltotalrate = $alltotalrate + $allrate[$nodename];
}

# ------------------------------------------------------------
# Scan master tree and exchange node numbers for average rates
# ------------------------------------------------------------
$_ = $master_tree;
my $key;
foreach $key (0..$numnodes[$gene]-1)
{
  #print $rate[$gene][$key] . "\n";
  s/(\:)($key)([,\)])/\:$allrate[$key]$3/
}

$ratogram = $_;
$allavrate = $alltotalrate / $numnodes[$gene];

print "Ratogram with rates averaged over all genes:\n" . $ratogram . "\n\n";
print "The number of branches is $numnodes[$gene], the average rate across the tree is $allavrate.\n";

# -----------------------------------
# Write final part of the output file
# -----------------------------------
print OF "tree all_genes_$allavrate = [&R] $ratogram\n";
print "The averaged ratogram has been written to the NEXUS file $outfile.\n";
print OF "end;";
close (OF) || warn "Error when closing output file: $!";
exit(0);


